@@ -14,9 +14,10 @@ import dataclasses
 import inspect
 import platform
 import sys
+import sysconfig
 import typing
 from functools import partial
-from typing import Any, ForwardRef, get_args
+from typing import Any, ForwardRef, List, Optional, get_args
 
 try:
     BaseExceptionGroup = BaseExceptionGroup
@@ -43,6 +44,8 @@ else:
 PYPY = platform.python_implementation() == "PyPy"
 GRAALPY = platform.python_implementation() == "GraalVM"
 WINDOWS = platform.system() == "Windows"
+# First defined in CPython 3.13, defaults to False
+FREE_THREADED_CPYTHON = bool(sysconfig.get_config_var("Py_GIL_DISABLED"))
 
 
 def add_note(exc, note):
@@ -50,7 +53,10 @@ def add_note(exc, note):
         exc.add_note(note)
     except AttributeError:
         if not hasattr(exc, "__notes__"):
-            exc.__notes__ = []
+            try:
+                exc.__notes__ = []
+            except AttributeError:
+                return  # give up, might be e.g. a frozen dataclass
         exc.__notes__.append(note)
 
 
@@ -180,6 +186,25 @@ def ceil(x):
     return y
 
 
+def extract_bits(x: int, /, width: Optional[int] = None) -> List[int]:
+    assert x >= 0
+    result = []
+    while x:
+        result.append(x & 1)
+        x >>= 1
+    if width is not None:
+        result = (result + [0] * width)[:width]
+    result.reverse()
+    return result
+
+
+# int.bit_count was added sometime around python 3.9
+try:
+    bit_count = int.bit_count
+except AttributeError:  # pragma: no cover
+    bit_count = lambda self: sum(extract_bits(abs(self)))
+
+
 def bad_django_TestCase(runner):
     if runner is None or "django.test" not in sys.modules:
         return False
